/* cache.c */

#include<time.h>
#include<string.h>
#include"cache.h"
#include"structures.h"	
#include"hash.h"
#include"cache_debug.h"
#include<linux/nfs.h>
#include<linux/sched.h>


/* struct list_node *block_if_lease_were_valid = NULL;*/

int read_cache(unsigned long inode_number, char *hostname,
	       off_t offset, int count, char *data, struct nfs_fattr *fattr)
/* 
   If the data requested is on the cache, makes the pointer data point to it
   and returns the lenght of the data, otherwise returns 0.
   Sets fattr which is used to determine if this version is up-to-date.
*/
{
  static int cache_initialized = 0;
  int size, last_byte;
  struct list_node *block;
  struct hash_table_entry *file;
  
  if (!cache_initialized) 
    {
      init_cache_table();
      cache_initialized = 1;
      return 0;
    }

  if (!(file  = find_hash_entry(inode_number, hostname)))
    return 0; /* this file is not cached */
  
   
  if(!(block = find_cache_block(file, offset)) ||
     block->offset + block->size <= offset)
    {
      fattr->mtime.seconds  = 0; /* The data aren't cached. */
      return 0;
    }
  
  fattr->blocksize      = 0; /* Means "do not update the old fattr". */
  fattr->mtime.seconds  = file->seconds;
  fattr->mtime.useconds = file->useconds;

  if (file->lease_expiration < CURRENT_TIME)
    /* The lease has expired but the data are cached. Let's ask for a new 
       lease exptime... The best thing to do would be making the RPC call here 
       and treat it here. But, to make things easier, we'll let 
       nfs_proc_read() do this job. */
      return 0;

/* At this point we have at least  some part of the data and the 
    lease for this file is not expired.
*/

  last_byte = block->offset + block->size;
  if (offset + count < last_byte) last_byte = offset + count;
  size = last_byte - offset;
  memcpy(data, block->data + (offset - block->offset), size);
    
  refresh_block_LRU_list(block);
  
  return size;

}/* read_cache */



int store_cache(unsigned long inode_number, char *hostname, off_t offset, 
		char *data, int len, unsigned long lease, 
		struct nfs_fattr *fattr)
/*
   Stores the data just nfs-readed in the cache.
*/
{
  /* static int NumeroDeBlocosRemovidos = 0; */
  
  struct list_node *block, *next;
  struct hash_table_entry *file = hash_entry(inode_number, hostname);

  if(!file) 
    {
      printk("store_cache: can't cache file (inode: %d)\n",inode_number);
      return 0;
    }

  if (file->lease_expiration < lease) 
    file->lease_expiration = lease;
  
  file->seconds  = fattr->mtime.seconds;
  file->useconds = fattr->mtime.useconds;

  block = find_predecessor(file, offset);
  if(!(block = new_cache_block(file, block))) return 0;
  
  block->file   = file;
  block->offset = offset;
  block->size   = len;
  if(!(block->data = cache_malloc(len))) return 0;
  memcpy(block->data, data, len);

  for(next  = block->offset_prev;            /* remove unnecessary blocks */
      next && (next->offset + next->size <= offset + len);
      next  = block->offset_prev)
    {
/*      printk("%d blocos removidos por informacao redundante\n",
	     ++NumeroDeBlocosRemovidos);
*/
      remove_cache_block_LRU_list(next);
      remove_cache_block_offset_list(next);
    }
  return 1;
}/* store_cache */



