/* hash.c: Finds and creates hash table entries */

#include <string.h>
#include "cache.h"
#include "structures.h"

extern struct hash_table_entry cache_table[HASH_SIZE];

struct hash_table_entry *find_hash_entry(unsigned long inode_number,
					 char *hostname)
/*
   Returns a pointer to the hash table entry of the file determined by
   the parameters.  If this entry doesn't exist, RETURNS NULL.
*/
{
  struct hash_table_entry *entry = &cache_table[hash_function(inode_number)];

  if(!entry->hostname) entry = entry->colision_next;
  while(entry &&
       (entry->inode_number != inode_number || strcmp(entry->hostname,hostname)))
    entry = entry->colision_next;

  return  (entry && entry->hostname) ? entry : NULL;
  
}/* find_hash_entry */




struct hash_table_entry *hash_entry(unsigned long inode_number, char *hostname)
/*
   Returns a pointer to the hash table entry of the file determined by the 
   parameters. If this entry doesn't exist, CREATES IT and returns the pointer.
*/
{
  int NotFound = 1; 
  struct hash_table_entry *aux;
  struct hash_table_entry *entry = &cache_table[hash_function(inode_number)];

  aux = entry;
  
  if(!entry->hostname && entry->colision_next) entry = entry->colision_next;

  while((NotFound=(!entry->hostname || entry->inode_number != inode_number || 
		   strcmp(entry->hostname, hostname))) &&
	entry->colision_next)
    entry = entry->colision_next;
  
  if (NotFound) /* The entry was not found. So, let's create one... */
    {
      if(aux->hostname) /* add a new cell after the cell pointed by entry */
	{
	  if(!(aux = cache_malloc(sizeof(struct hash_table_entry)))) 
	    return NULL;
	  entry->colision_next = aux;
	  aux->colision_prev   = entry;
	  aux->colision_next   = NULL;
	}

      if(!(aux->hostname = cache_malloc(strlen(hostname)+1))) return NULL;
      strcpy(aux->hostname,hostname);
      aux->inode_number = inode_number;
      aux->offset_list  = NULL;
      entry = aux;    
    }
  
  return entry; 

}/* hash_entry */



